// Copyright (c) 2013, the Dart project authors.  Please see the AUTHORS file
// for details. All rights reserved. Use of this source code is governed by a
// BSD-style license that can be found in the LICENSE file.

library quiver.async;

import 'dart:async';

part 'src/async/future_group.dart';

/**
 * Calls [action] for each item in [iterable] in turn, waiting for the Future
 * returned by action to complete.
 *
 * If the Future completes to [true], iteration continues.
 *
 * The Future returned completes to [true] if the entire iterable was processed,
 * otherwise [false].
 */
Future doWhileAsync(Iterable iterable, Future<bool> action(i)) =>
    _doWhileAsync(iterable.iterator, action);

Future _doWhileAsync(Iterator iterator, Future<bool> action(i)) =>
  (iterator.moveNext())
      ? action(iterator.current).then((bool result) =>
        (result)
            ? _doWhileAsync(iterator, action)
            : new Future.value(false))
      : new Future.value(true);

/**
 * Reduces a collection to a single value by iteratively combining elements
 * of the collection using the provided [combine] function. Similar to
 * [Iterable.reduce], except that [combine] is an async function that returns a
 * [Future].
 */
Future reduceAsync(Iterable iterable, initialValue, combine(previous, element))
    => _reduceAsync(iterable.iterator, initialValue, combine);

Future _reduceAsync(Iterator iterator, currentValue,
                    combine(previous, element)) {
  if (iterator.moveNext()) {
    return combine(currentValue, iterator.current).then((result) =>
        _reduceAsync(iterator, result, combine));
  }
  return new Future.value(currentValue);
}
